@@ -11,17 +11,22 @@ from .base import LevelDir
 logg = logging.getLogger(__name__)
 
 
+def default_formatter(hx):
+    return hx.upper()
+
+
 class HexDir(LevelDir):
 
-    def __init__(self, root_path, key_length, levels=2, prefix_length=0):
+    def __init__(self, root_path, key_length, levels=2, prefix_length=0, formatter=default_formatter):
         super(HexDir, self).__init__(root_path, levels, key_length + prefix_length)
         #self.path = root_path
         self.key_length = key_length
         self.prefix_length = prefix_length
         self.__levels = levels + 2
+        self.formatter = formatter
 
 
-    def add(self, key, content, prefix=b''):
+    def __check(self, key, content, prefix):
         l = len(key)
         if l != self.key_length:
             raise ValueError('expected key length {}, got {}'.format(self.key_length, l))
@@ -32,9 +37,16 @@ class HexDir(LevelDir):
             raise ValueError('content must be bytes, got {}'.format(type(content).__name__))
         if prefix != None and not isinstance(prefix, bytes):
             raise ValueError('prefix must be bytes, got {}'.format(type(content).__name__))
+
+
+    def add(self, key, content, prefix=b''):
+        self.__check(key, content, prefix)
         key_hex = key.hex()
         entry_path = self.to_filepath(key_hex)
+        return self.__add(entry_path, key, content, key_hex, prefix=prefix)
 
+
+    def __add(self, entry_path, key, content, display_key, prefix=b''):
         c = self.count()
 
         os.makedirs(os.path.dirname(entry_path), exist_ok=True)
@@ -48,11 +60,19 @@ class HexDir(LevelDir):
         f.write(key)
         f.close()
 
-        logg.debug('created new hexdir entry {} idx {} in {}'.format(key_hex, c, entry_path)) 
+        logg.debug('created new hexdir entry {} idx {} in {}'.format(display_key, c, entry_path)) 
     
         return (c, entry_path)
 
 
+    def add_dir(self, file_key, key, content, prefix=b''):
+        self.__check(key, content, prefix)
+        key_hex = key.hex()
+        entry_path = self.to_filepath(key_hex)
+        entry_path = os.path.join(entry_path, file_key)
+        return self.__add(entry_path, key, content, key_hex, prefix=prefix)
+
+
     def __cursor(self, idx):
         return idx * (self.prefix_length + self.key_length)
 
@@ -84,7 +104,7 @@ class HexDir(LevelDir):
         lead = ''
         for i in range(0, self.__levels, 2):
             lead += hx[i:i+2] + '/'
-        return lead.upper()
+        return self.formatter(lead)
 
 
     def to_dirpath(self, hx):
@@ -94,14 +114,5 @@ class HexDir(LevelDir):
 
     def to_filepath(self, hx):
         dir_path = self.to_dirpath(hx)
-        file_path = os.path.join(dir_path, hx.upper())
+        file_path = os.path.join(dir_path, self.formatter(hx))
         return file_path
-
-
-    @staticmethod
-    def __prepare_directory(path):
-        os.makedirs(path, exist_ok=True)
-        state_file = os.path.join(path, 'master')
-        f = open(state_file, 'w')
-        f.close()
-
